from typing import overload
from enum import Enum
import typing

import QuantConnect.Optimizer
import QuantConnect.Optimizer.Launcher
import QuantConnect.Optimizer.Parameters
import System


class Program(System.Object):
    """This class has no documentation."""

    @staticmethod
    def main(args: typing.List[str]) -> None:
        ...


class ConsoleLeanOptimizer(QuantConnect.Optimizer.LeanOptimizer):
    """Optimizer implementation that launches Lean as a local process"""

    def __init__(self, nodePacket: QuantConnect.Optimizer.OptimizationNodePacket) -> None:
        """
        Creates a new instance
        
        :param nodePacket: The optimization node packet to handle
        """
        ...

    def abort_lean(self, backtest_id: str) -> None:
        """
        Stops lean process
        
        This method is protected.
        
        :param backtest_id: Specified backtest id
        """
        ...

    def run_lean(self, parameter_set: QuantConnect.Optimizer.Parameters.ParameterSet, backtest_name: str) -> str:
        """
        Handles starting Lean for a given parameter set
        
        This method is protected.
        
        :param parameter_set: The parameter set for the backtest to run
        :param backtest_name: The backtest name to use
        :returns: The new unique backtest id.
        """
        ...

    def send_update(self) -> None:
        """
        Sends an update of the current optimization status to the user
        
        This method is protected.
        """
        ...





from typing import overload
from enum import Enum
import datetime
import typing

import QuantConnect.Orders
import QuantConnect.Orders.TimeInForces
import QuantConnect.Securities


class GoodTilDateTimeInForce(QuantConnect.Orders.TimeInForce):
    """Good Til Date Time In Force - order expires and will be cancelled on a fixed date/time"""

    @property
    def expiry(self) -> datetime.datetime:
        """The date/time on which the order will expire and will be cancelled"""
        ...

    def __init__(self, expiry: typing.Union[datetime.datetime, datetime.date]) -> None:
        """Initializes a new instance of the GoodTilDateTimeInForce class"""
        ...

    def get_forex_order_expiry_date_time(self, order: QuantConnect.Orders.Order) -> datetime.datetime:
        """Returns the expiry date and time (UTC) for a Forex order"""
        ...

    def is_fill_valid(self, security: QuantConnect.Securities.Security, order: QuantConnect.Orders.Order, fill: QuantConnect.Orders.OrderEvent) -> bool:
        """
        Checks if an order fill is valid
        
        :param security: The security matching the order
        :param order: The order to be checked
        :param fill: The order fill to be checked
        :returns: Returns true if the order fill can be emitted, false otherwise.
        """
        ...

    def is_order_expired(self, security: QuantConnect.Securities.Security, order: QuantConnect.Orders.Order) -> bool:
        """
        Checks if an order is expired
        
        :param security: The security matching the order
        :param order: The order to be checked
        :returns: Returns true if the order has expired, false otherwise.
        """
        ...


class GoodTilCanceledTimeInForce(QuantConnect.Orders.TimeInForce):
    """Good Til Canceled Time In Force - order does never expires"""

    def is_fill_valid(self, security: QuantConnect.Securities.Security, order: QuantConnect.Orders.Order, fill: QuantConnect.Orders.OrderEvent) -> bool:
        """
        Checks if an order fill is valid
        
        :param security: The security matching the order
        :param order: The order to be checked
        :param fill: The order fill to be checked
        :returns: Returns true if the order fill can be emitted, false otherwise.
        """
        ...

    def is_order_expired(self, security: QuantConnect.Securities.Security, order: QuantConnect.Orders.Order) -> bool:
        """
        Checks if an order is expired
        
        :param security: The security matching the order
        :param order: The order to be checked
        :returns: Returns true if the order has expired, false otherwise.
        """
        ...


class DayTimeInForce(QuantConnect.Orders.TimeInForce):
    """Day Time In Force - order expires at market close"""

    def is_fill_valid(self, security: QuantConnect.Securities.Security, order: QuantConnect.Orders.Order, fill: QuantConnect.Orders.OrderEvent) -> bool:
        """
        Checks if an order fill is valid
        
        :param security: The security matching the order
        :param order: The order to be checked
        :param fill: The order fill to be checked
        :returns: Returns true if the order fill can be emitted, false otherwise.
        """
        ...

    def is_order_expired(self, security: QuantConnect.Securities.Security, order: QuantConnect.Orders.Order) -> bool:
        """
        Checks if an order is expired
        
        :param security: The security matching the order
        :param order: The order to be checked
        :returns: Returns true if the order has expired, false otherwise.
        """
        ...





from typing import overload
from enum import Enum
import typing

import System
import System.Collections.Generic
import System.IO
import System.Reflection
import System.Runtime.Loader

System_Runtime_Loader__EventContainer_Callable = typing.TypeVar("System_Runtime_Loader__EventContainer_Callable")
System_Runtime_Loader__EventContainer_ReturnType = typing.TypeVar("System_Runtime_Loader__EventContainer_ReturnType")


class AssemblyDependencyResolver(System.Object):
    """This class has no documentation."""

    @overload
    def __init__(self, componentAssemblyPath: str) -> None:
        ...

    @overload
    def __init__(self, componentAssemblyPath: str) -> None:
        ...

    @overload
    def resolve_assembly_to_path(self, assembly_name: System.Reflection.AssemblyName) -> str:
        ...

    @overload
    def resolve_assembly_to_path(self, assembly_name: System.Reflection.AssemblyName) -> str:
        ...

    @overload
    def resolve_unmanaged_dll_to_path(self, unmanaged_dll_name: str) -> str:
        ...

    @overload
    def resolve_unmanaged_dll_to_path(self, unmanaged_dll_name: str) -> str:
        ...


class AssemblyLoadContext(System.Object):
    """This class has no documentation."""

    class ContextualReflectionScope(System.IDisposable):
        """Opaque disposable struct used to restore CurrentContextualReflectionContext"""

        def dispose(self) -> None:
            ...

    @property
    def assemblies(self) -> System.Collections.Generic.IEnumerable[System.Reflection.Assembly]:
        ...

    @property
    def resolving_unmanaged_dll(self) -> _EventContainer[typing.Callable[[System.Reflection.Assembly, str], System.IntPtr], System.IntPtr]:
        ...

    @property
    def resolving(self) -> _EventContainer[typing.Callable[[System.Runtime.Loader.AssemblyLoadContext, System.Reflection.AssemblyName], System.Reflection.Assembly], System.Reflection.Assembly]:
        ...

    @property
    def unloading(self) -> _EventContainer[typing.Callable[[System.Runtime.Loader.AssemblyLoadContext], None], None]:
        ...

    DEFAULT: System.Runtime.Loader.AssemblyLoadContext

    @property
    def is_collectible(self) -> bool:
        ...

    @property
    def name(self) -> str:
        ...

    ALL: System.Collections.Generic.IEnumerable[System.Runtime.Loader.AssemblyLoadContext]

    CURRENT_CONTEXTUAL_REFLECTION_CONTEXT: System.Runtime.Loader.AssemblyLoadContext
    """Nullable current AssemblyLoadContext used for context sensitive reflection APIs"""

    @overload
    def __init__(self) -> None:
        """This method is protected."""
        ...

    @overload
    def __init__(self, isCollectible: bool) -> None:
        """This method is protected."""
        ...

    @overload
    def __init__(self, name: str, isCollectible: bool = False) -> None:
        ...

    @overload
    def enter_contextual_reflection(self) -> System.Runtime.Loader.AssemblyLoadContext.ContextualReflectionScope:
        """
        Enter scope using this AssemblyLoadContext for ContextualReflection
        
        :returns: A disposable ContextualReflectionScope for use in a using block.
        """
        ...

    @staticmethod
    @overload
    def enter_contextual_reflection(activating: System.Reflection.Assembly) -> System.Runtime.Loader.AssemblyLoadContext.ContextualReflectionScope:
        """
        Enter scope using this AssemblyLoadContext for ContextualReflection
        
        :param activating: Set CurrentContextualReflectionContext to the AssemblyLoadContext which loaded activating.
        :returns: A disposable ContextualReflectionScope for use in a using block.
        """
        ...

    @staticmethod
    def get_assembly_name(assembly_path: str) -> System.Reflection.AssemblyName:
        ...

    @staticmethod
    def get_load_context(assembly: System.Reflection.Assembly) -> System.Runtime.Loader.AssemblyLoadContext:
        ...

    def load(self, assembly_name: System.Reflection.AssemblyName) -> System.Reflection.Assembly:
        """This method is protected."""
        ...

    def load_from_assembly_name(self, assembly_name: System.Reflection.AssemblyName) -> System.Reflection.Assembly:
        ...

    def load_from_assembly_path(self, assembly_path: str) -> System.Reflection.Assembly:
        ...

    def load_from_native_image_path(self, native_image_path: str, assembly_path: str) -> System.Reflection.Assembly:
        ...

    @overload
    def load_from_stream(self, assembly: System.IO.Stream) -> System.Reflection.Assembly:
        ...

    @overload
    def load_from_stream(self, assembly: System.IO.Stream, assembly_symbols: System.IO.Stream) -> System.Reflection.Assembly:
        ...

    def load_unmanaged_dll(self, unmanaged_dll_name: str) -> System.IntPtr:
        """This method is protected."""
        ...

    def load_unmanaged_dll_from_path(self, unmanaged_dll_path: str) -> System.IntPtr:
        """This method is protected."""
        ...

    def set_profile_optimization_root(self, directory_path: str) -> None:
        ...

    def start_profile_optimization(self, profile: str) -> None:
        ...

    def to_string(self) -> str:
        ...

    def unload(self) -> None:
        ...


class _EventContainer(typing.Generic[System_Runtime_Loader__EventContainer_Callable, System_Runtime_Loader__EventContainer_ReturnType]):
    """This class is used to provide accurate autocomplete on events and cannot be imported."""

    def __call__(self, *args: typing.Any, **kwargs: typing.Any) -> System_Runtime_Loader__EventContainer_ReturnType:
        """Fires the event."""
        ...

    def __iadd__(self, item: System_Runtime_Loader__EventContainer_Callable) -> None:
        """Registers an event handler."""
        ...

    def __isub__(self, item: System_Runtime_Loader__EventContainer_Callable) -> None:
        """Unregisters an event handler."""
        ...





from typing import overload
from enum import Enum
import abc

import QuantConnect.Interfaces
import QuantConnect.Orders
import QuantConnect.Orders.Slippage
import QuantConnect.Securities
import System


class ISlippageModel(metaclass=abc.ABCMeta):
    """Represents a model that simulates market order slippage"""

    def get_slippage_approximation(self, asset: QuantConnect.Securities.Security, order: QuantConnect.Orders.Order) -> float:
        """Slippage Model. Return a decimal cash slippage approximation on the order."""
        ...


class VolumeShareSlippageModel(System.Object, QuantConnect.Orders.Slippage.ISlippageModel):
    """
    Represents a slippage model that is calculated by multiplying the price impact constant
    by the square of the ratio of the order to the total volume.
    """

    def __init__(self, volumeLimit: float = 0.025, priceImpact: float = 0.1) -> None:
        """
        Initializes a new instance of the VolumeShareSlippageModel class
        
        :param priceImpact: Defines how large of an impact the order will have on the price calculation
        """
        ...

    def get_slippage_approximation(self, asset: QuantConnect.Securities.Security, order: QuantConnect.Orders.Order) -> float:
        """Slippage Model. Return a decimal cash slippage approximation on the order."""
        ...


class MarketImpactSlippageModel(System.Object, QuantConnect.Orders.Slippage.ISlippageModel):
    """
    Slippage model that mimic the effect brought by market impact,
    i.e. consume the volume listed in the order book
    """

    def __init__(self, algorithm: QuantConnect.Interfaces.IAlgorithm, nonNegative: bool = True, latency: float = ..., impactTime: float = ..., alpha: float = ..., beta: float = ..., gamma: float = ..., eta: float = ..., delta: float = ..., randomSeed: int = 50) -> None:
        """
        Instantiate a new instance of MarketImpactSlippageModel
        
        :param algorithm: IAlgorithm instance
        :param nonNegative: Indicator whether only non-negative slippage allowed
        :param latency: Time between order submitted and filled, in seconds(s)
        :param impactTime: Time between order filled and new equilibrium established, in second(s)
        :param alpha: Exponent of the permanent impact function
        :param beta: Exponent of the temporary impact function
        :param gamma: Coefficient of the permanent impact function
        :param eta: Coefficient of the temporary impact function
        :param delta: Liquidity scaling factor for permanent impact
        :param randomSeed: Random seed for generating gaussian noise
        """
        ...

    def get_slippage_approximation(self, asset: QuantConnect.Securities.Security, order: QuantConnect.Orders.Order) -> float:
        """Slippage Model. Return a decimal cash slippage approximation on the order."""
        ...


class NullSlippageModel(System.Object, QuantConnect.Orders.Slippage.ISlippageModel):
    """Null slippage model, which provider no slippage"""

    INSTANCE: QuantConnect.Orders.Slippage.NullSlippageModel
    """The null slippage model instance"""

    def get_slippage_approximation(self, asset: QuantConnect.Securities.Security, order: QuantConnect.Orders.Order) -> float:
        """Will return no slippage"""
        ...


class ConstantSlippageModel(System.Object, QuantConnect.Orders.Slippage.ISlippageModel):
    """Represents a slippage model that uses a constant percentage of slip"""

    def __init__(self, slippagePercent: float) -> None:
        """
        Initializes a new instance of the ConstantSlippageModel class
        
        :param slippagePercent: The slippage percent for each order. Percent is ranged 0 to 1.
        """
        ...

    def get_slippage_approximation(self, asset: QuantConnect.Securities.Security, order: QuantConnect.Orders.Order) -> float:
        """Slippage Model. Return a decimal cash slippage approximation on the order."""
        ...


class AlphaStreamsSlippageModel(System.Object, QuantConnect.Orders.Slippage.ISlippageModel):
    """Represents a slippage model that uses a constant percentage of slip"""

    def __init__(self) -> None:
        """Initializes a new instance of the AlphaStreamsSlippageModel class"""
        ...

    def get_slippage_approximation(self, asset: QuantConnect.Securities.Security, order: QuantConnect.Orders.Order) -> float:
        """Return a decimal cash slippage approximation on the order."""
        ...





from typing import overload
from enum import Enum
import QuantConnect.DownloaderDataProvider.Launcher.Models.Constants
import System


class DownloaderCommandArguments(System.Object):
    """This class has no documentation."""

    COMMAND_DOWNLOADER_DATA_DOWNLOADER: str = "data-downloader"

    COMMAND_DATA_TYPE: str = "data-type"

    COMMAND_TICKERS: str = "tickers"

    COMMAND_SECURITY_TYPE: str = "security-type"

    COMMAND_MARKET_NAME: str = "market"

    COMMAND_RESOLUTION: str = "resolution"

    COMMAND_START_DATE: str = "start-date"

    COMMAND_END_DATE: str = "end-date"





from typing import overload
from enum import Enum
import abc

import System
import System.Security
import System.Security.Permissions


class PermissionState(Enum):
    """This class has no documentation."""

    NONE = 0

    UNRESTRICTED = 1


class SecurityAction(Enum):
    """Obsoletions.CodeAccessSecurityMessage"""

    ASSERT = 3

    DEMAND = 2

    DENY = 4

    INHERITANCE_DEMAND = 7

    LINK_DEMAND = 6

    PERMIT_ONLY = 5

    REQUEST_MINIMUM = 8

    REQUEST_OPTIONAL = 9

    REQUEST_REFUSE = 10


class SecurityAttribute(System.Attribute, metaclass=abc.ABCMeta):
    """Obsoletions.CodeAccessSecurityMessage"""

    @property
    def action(self) -> System.Security.Permissions.SecurityAction:
        ...

    @property.setter
    def action(self, value: System.Security.Permissions.SecurityAction) -> None:
        ...

    @property
    def unrestricted(self) -> bool:
        ...

    @property.setter
    def unrestricted(self, value: bool) -> None:
        ...

    def __init__(self, action: System.Security.Permissions.SecurityAction) -> None:
        """This method is protected."""
        ...

    def create_permission(self) -> System.Security.IPermission:
        ...


class CodeAccessSecurityAttribute(System.Security.Permissions.SecurityAttribute, metaclass=abc.ABCMeta):
    """Obsoletions.CodeAccessSecurityMessage"""

    def __init__(self, action: System.Security.Permissions.SecurityAction) -> None:
        """This method is protected."""
        ...


class SecurityPermissionFlag(Enum):
    """Obsoletions.CodeAccessSecurityMessage"""

    ALL_FLAGS = 16383

    ASSERTION = 1

    BINDING_REDIRECTS = 8192

    CONTROL_APP_DOMAIN = 1024

    CONTROL_DOMAIN_POLICY = 256

    CONTROL_EVIDENCE = 32

    CONTROL_POLICY = 64

    CONTROL_PRINCIPAL = 512

    CONTROL_THREAD = 16

    EXECUTION = 8

    INFRASTRUCTURE = 4096

    NO_FLAGS = 0

    REMOTING_CONFIGURATION = 2048

    SERIALIZATION_FORMATTER = 128

    SKIP_VERIFICATION = 4

    UNMANAGED_CODE = 2


class SecurityPermissionAttribute(System.Security.Permissions.CodeAccessSecurityAttribute):
    """Obsoletions.CodeAccessSecurityMessage"""

    @property
    def assertion(self) -> bool:
        ...

    @property.setter
    def assertion(self, value: bool) -> None:
        ...

    @property
    def binding_redirects(self) -> bool:
        ...

    @property.setter
    def binding_redirects(self, value: bool) -> None:
        ...

    @property
    def control_app_domain(self) -> bool:
        ...

    @property.setter
    def control_app_domain(self, value: bool) -> None:
        ...

    @property
    def control_domain_policy(self) -> bool:
        ...

    @property.setter
    def control_domain_policy(self, value: bool) -> None:
        ...

    @property
    def control_evidence(self) -> bool:
        ...

    @property.setter
    def control_evidence(self, value: bool) -> None:
        ...

    @property
    def control_policy(self) -> bool:
        ...

    @property.setter
    def control_policy(self, value: bool) -> None:
        ...

    @property
    def control_principal(self) -> bool:
        ...

    @property.setter
    def control_principal(self, value: bool) -> None:
        ...

    @property
    def control_thread(self) -> bool:
        ...

    @property.setter
    def control_thread(self, value: bool) -> None:
        ...

    @property
    def execution(self) -> bool:
        ...

    @property.setter
    def execution(self, value: bool) -> None:
        ...

    @property
    def flags(self) -> System.Security.Permissions.SecurityPermissionFlag:
        ...

    @property.setter
    def flags(self, value: System.Security.Permissions.SecurityPermissionFlag) -> None:
        ...

    @property
    def infrastructure(self) -> bool:
        ...

    @property.setter
    def infrastructure(self, value: bool) -> None:
        ...

    @property
    def remoting_configuration(self) -> bool:
        ...

    @property.setter
    def remoting_configuration(self, value: bool) -> None:
        ...

    @property
    def serialization_formatter(self) -> bool:
        ...

    @property.setter
    def serialization_formatter(self, value: bool) -> None:
        ...

    @property
    def skip_verification(self) -> bool:
        ...

    @property.setter
    def skip_verification(self, value: bool) -> None:
        ...

    @property
    def unmanaged_code(self) -> bool:
        ...

    @property.setter
    def unmanaged_code(self, value: bool) -> None:
        ...

    def __init__(self, action: System.Security.Permissions.SecurityAction) -> None:
        ...

    def create_permission(self) -> System.Security.IPermission:
        ...





from typing import overload
from enum import Enum
import datetime
import typing

import QuantConnect
import QuantConnect.DownloaderDataProvider.Launcher
import QuantConnect.Interfaces
import System
import System.Collections.Generic


class DownloaderDataProviderArgumentParser(System.Object):
    """This class has no documentation."""

    @staticmethod
    def parse_arguments(args: typing.List[str]) -> System.Collections.Generic.Dictionary[str, System.Object]:
        """
        Parses the command-line arguments and returns a dictionary containing parsed values.
        
        :param args: An array of command-line arguments.
        :returns: A dictionary containing parsed values from the command-line arguments.
        """
        ...


class DataDownloadConfig:
    """Represents the configuration for data download."""

    @property
    def tick_type(self) -> QuantConnect.TickType:
        """Type of tick data to download."""
        ...

    @property
    def security_type(self) -> QuantConnect.SecurityType:
        """Type of security for which data is to be downloaded."""
        ...

    @property
    def resolution(self) -> QuantConnect.Resolution:
        """Resolution of the downloaded data."""
        ...

    @property
    def start_date(self) -> datetime.datetime:
        """Start date for the data download."""
        ...

    @property
    def end_date(self) -> datetime.datetime:
        """End date for the data download."""
        ...

    @property
    def market_name(self) -> str:
        """Market name for which the data is to be downloaded."""
        ...

    @property
    def symbols(self) -> System.Collections.Generic.List[QuantConnect.Symbol]:
        """List of symbols for which data is to be downloaded."""
        ...

    @overload
    def __init__(self) -> None:
        """Initializes a new instance of the DataDownloadConfig struct."""
        ...

    @overload
    def __init__(self, tickType: QuantConnect.TickType, securityType: QuantConnect.SecurityType, resolution: QuantConnect.Resolution, startDate: typing.Union[datetime.datetime, datetime.date], endDate: typing.Union[datetime.datetime, datetime.date], market: str, symbols: System.Collections.Generic.List[QuantConnect.Symbol]) -> None:
        """
        Initializes a new instance of the DataDownloadConfig class with the specified parameters.
        
        :param tickType: The type of tick data to be downloaded.
        :param securityType: The type of security for which data is being downloaded.
        :param resolution: The resolution of the data being downloaded.
        :param startDate: The start date for the data download range.
        :param endDate: The end date for the data download range.
        :param market: The name of the market from which the data is being downloaded.
        :param symbols: A list of symbols for which data is being downloaded.
        """
        ...

    def to_string(self) -> str:
        """
        Returns a string representation of the DataDownloadConfig struct.
        
        :returns: A string representation of the DataDownloadConfig struct.
        """
        ...


class Program(System.Object):
    """This class has no documentation."""

    @staticmethod
    def initialize_configurations() -> None:
        """
        Initializes various configurations for the application.
        This method sets up logging, data providers, map file providers, and factor file providers.
        """
        ...

    @staticmethod
    def main(args: typing.List[str]) -> None:
        """
        The main entry point for the application.
        
        :param args: Command-line arguments passed to the application.
        """
        ...

    @staticmethod
    def run_download(data_downloader: QuantConnect.IDataDownloader, data_download_config: QuantConnect.DownloaderDataProvider.Launcher.DataDownloadConfig, data_directory: str, data_cache_provider: QuantConnect.Interfaces.IDataCacheProvider, map_symbol: bool = True) -> None:
        """
        Executes a data download operation using the specified data downloader.
        
        :param data_downloader: An instance of an object implementing the IDataDownloader interface, responsible for downloading data.
        :param data_download_config: Configuration settings for the data download operation.
        :param data_directory: The directory where the downloaded data will be stored.
        :param data_cache_provider: The provider used to cache history data files
        :param map_symbol: True if the symbol should be mapped while writing the data
        """
        ...





from typing import overload
from enum import Enum
import abc
import datetime
import typing

import QuantConnect
import QuantConnect.Interfaces
import QuantConnect.Lean.Engine.DataFeeds
import QuantConnect.Lean.Engine.RealTime
import QuantConnect.Lean.Engine.Results
import QuantConnect.Lean.Engine.Setup
import QuantConnect.Lean.Engine.TransactionHandlers
import QuantConnect.Packets
import QuantConnect.Util
import System
import System.Collections.Generic


class SetupHandlerParameters(System.Object):
    """Defines the parameters for ISetupHandler"""

    @property
    def universe_selection(self) -> QuantConnect.Lean.Engine.DataFeeds.UniverseSelection:
        """Gets the universe selection"""
        ...

    @property
    def algorithm(self) -> QuantConnect.Interfaces.IAlgorithm:
        """Gets the algorithm"""
        ...

    @property
    def brokerage(self) -> QuantConnect.Interfaces.IBrokerage:
        """Gets the Brokerage"""
        ...

    @property
    def algorithm_node_packet(self) -> QuantConnect.Packets.AlgorithmNodePacket:
        """Gets the algorithm node packet"""
        ...

    @property
    def result_handler(self) -> QuantConnect.Lean.Engine.Results.IResultHandler:
        """Gets the algorithm node packet"""
        ...

    @property
    def transaction_handler(self) -> QuantConnect.Lean.Engine.TransactionHandlers.ITransactionHandler:
        """Gets the TransactionHandler"""
        ...

    @property
    def real_time_handler(self) -> QuantConnect.Lean.Engine.RealTime.IRealTimeHandler:
        """Gets the RealTimeHandler"""
        ...

    @property
    def data_cache_provider(self) -> QuantConnect.Interfaces.IDataCacheProvider:
        """Gets the DataCacheProvider"""
        ...

    @property
    def map_file_provider(self) -> QuantConnect.Interfaces.IMapFileProvider:
        """The map file provider instance of the algorithm"""
        ...

    def __init__(self, universeSelection: QuantConnect.Lean.Engine.DataFeeds.UniverseSelection, algorithm: QuantConnect.Interfaces.IAlgorithm, brokerage: QuantConnect.Interfaces.IBrokerage, algorithmNodePacket: QuantConnect.Packets.AlgorithmNodePacket, resultHandler: QuantConnect.Lean.Engine.Results.IResultHandler, transactionHandler: QuantConnect.Lean.Engine.TransactionHandlers.ITransactionHandler, realTimeHandler: QuantConnect.Lean.Engine.RealTime.IRealTimeHandler, dataCacheProvider: QuantConnect.Interfaces.IDataCacheProvider, mapFileProvider: QuantConnect.Interfaces.IMapFileProvider) -> None:
        """
        Creates a new instance
        
        :param universeSelection: The universe selection instance
        :param algorithm: Algorithm instance
        :param brokerage: New brokerage output instance
        :param algorithmNodePacket: Algorithm job task
        :param resultHandler: The configured result handler
        :param transactionHandler: The configured transaction handler
        :param realTimeHandler: The configured real time handler
        :param dataCacheProvider: The configured data cache provider
        :param mapFileProvider: The map file provider
        """
        ...


class ISetupHandler(System.IDisposable, metaclass=abc.ABCMeta):
    """Interface to setup the algorithm. Pass in a raw algorithm, return one with portfolio, cash, etc already preset."""

    @property
    @abc.abstractmethod
    def worker_thread(self) -> QuantConnect.Util.WorkerThread:
        """The worker thread instance the setup handler should use"""
        ...

    @property.setter
    def worker_thread(self, value: QuantConnect.Util.WorkerThread) -> None:
        ...

    @property
    @abc.abstractmethod
    def errors(self) -> System.Collections.Generic.List[System.Exception]:
        """Any errors from the initialization stored here:"""
        ...

    @property.setter
    def errors(self, value: System.Collections.Generic.List[System.Exception]) -> None:
        ...

    @property
    @abc.abstractmethod
    def maximum_runtime(self) -> datetime.timedelta:
        """Get the maximum runtime for this algorithm job."""
        ...

    @property
    @abc.abstractmethod
    def starting_portfolio_value(self) -> float:
        """Algorithm starting capital for statistics calculations"""
        ...

    @property
    @abc.abstractmethod
    def starting_date(self) -> datetime.datetime:
        """Start date for analysis loops to search for data."""
        ...

    @property
    @abc.abstractmethod
    def max_orders(self) -> int:
        """Maximum number of orders for the algorithm run -- applicable for backtests only."""
        ...

    def create_algorithm_instance(self, algorithm_node_packet: QuantConnect.Packets.AlgorithmNodePacket, assembly_path: str) -> QuantConnect.Interfaces.IAlgorithm:
        """
        Create a new instance of an algorithm from a physical dll path.
        
        :param algorithm_node_packet: Details of the task required
        :param assembly_path: The path to the assembly's location
        :returns: A new instance of IAlgorithm, or throws an exception if there was an error.
        """
        ...

    def create_brokerage(self, algorithm_node_packet: QuantConnect.Packets.AlgorithmNodePacket, uninitialized_algorithm: QuantConnect.Interfaces.IAlgorithm, factory: typing.Optional[QuantConnect.Interfaces.IBrokerageFactory]) -> typing.Union[QuantConnect.Interfaces.IBrokerage, QuantConnect.Interfaces.IBrokerageFactory]:
        """
        Creates the brokerage as specified by the job packet
        
        :param algorithm_node_packet: Job packet
        :param uninitialized_algorithm: The algorithm instance before Initialize has been called
        :param factory: The brokerage factory
        :returns: The brokerage instance, or throws if error creating instance.
        """
        ...

    def setup(self, parameters: QuantConnect.Lean.Engine.Setup.SetupHandlerParameters) -> bool:
        """
        Primary entry point to setup a new algorithm
        
        :param parameters: The parameters object to use
        :returns: True on successfully setting up the algorithm state, or false on error.
        """
        ...


class BacktestingSetupHandler(System.Object, QuantConnect.Lean.Engine.Setup.ISetupHandler):
    """Backtesting setup handler processes the algorithm initialize method and sets up the internal state of the algorithm class."""

    @property
    def worker_thread(self) -> QuantConnect.Util.WorkerThread:
        """The worker thread instance the setup handler should use"""
        ...

    @property.setter
    def worker_thread(self, value: QuantConnect.Util.WorkerThread) -> None:
        ...

    @property
    def errors(self) -> System.Collections.Generic.List[System.Exception]:
        """Internal errors list from running the setup procedures."""
        ...

    @property.setter
    def errors(self, value: System.Collections.Generic.List[System.Exception]) -> None:
        ...

    @property
    def maximum_runtime(self) -> datetime.timedelta:
        """Maximum runtime of the algorithm in seconds."""
        ...

    @property
    def starting_portfolio_value(self) -> float:
        """Starting capital according to the users initialize routine."""
        ...

    @property
    def starting_date(self) -> datetime.datetime:
        """Start date for analysis loops to search for data."""
        ...

    @property
    def max_orders(self) -> int:
        """Maximum number of orders for this backtest."""
        ...

    def __init__(self) -> None:
        """Initialize the backtest setup handler."""
        ...

    def create_algorithm_instance(self, algorithm_node_packet: QuantConnect.Packets.AlgorithmNodePacket, assembly_path: str) -> QuantConnect.Interfaces.IAlgorithm:
        """
        Create a new instance of an algorithm from a physical dll path.
        
        :param algorithm_node_packet: Details of the task required
        :param assembly_path: The path to the assembly's location
        :returns: A new instance of IAlgorithm, or throws an exception if there was an error.
        """
        ...

    def create_brokerage(self, algorithm_node_packet: QuantConnect.Packets.AlgorithmNodePacket, uninitialized_algorithm: QuantConnect.Interfaces.IAlgorithm, factory: typing.Optional[QuantConnect.Interfaces.IBrokerageFactory]) -> typing.Union[QuantConnect.Interfaces.IBrokerage, QuantConnect.Interfaces.IBrokerageFactory]:
        """
        Creates a new BacktestingBrokerage instance
        
        :param algorithm_node_packet: Job packet
        :param uninitialized_algorithm: The algorithm instance before Initialize has been called
        :param factory: The brokerage factory
        :returns: The brokerage instance, or throws if error creating instance.
        """
        ...

    def dispose(self) -> None:
        """Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources."""
        ...

    def setup(self, parameters: QuantConnect.Lean.Engine.Setup.SetupHandlerParameters) -> bool:
        """
        Setup the algorithm cash, dates and data subscriptions as desired.
        
        :param parameters: The parameters object to use
        :returns: Boolean true on successfully initializing the algorithm.
        """
        ...


class ConsoleSetupHandler(QuantConnect.Lean.Engine.Setup.BacktestingSetupHandler):
    """
    Kept for backwards compatibility-
    
    Should use BacktestingSetupHandler instead
    """


class AlgorithmSetupException(System.Exception):
    """Defines an exception generated in the course of invoking ISetupHandler.Setup"""

    @overload
    def __init__(self, message: str) -> None:
        """
        Initializes a new instance of the AlgorithmSetupException class
        
        :param message: The error message
        """
        ...

    @overload
    def __init__(self, message: str, inner: System.Exception) -> None:
        """
        Initializes a new instance of the AlgorithmSetupException class
        
        :param message: The error message
        :param inner: The inner exception being wrapped
        """
        ...


class BrokerageSetupHandler(System.Object, QuantConnect.Lean.Engine.Setup.ISetupHandler):
    """Defines a set up handler that initializes the algorithm instance using values retrieved from the user's brokerage account"""

    max_allocation_limit_config: str = "max-allocation-limit"
    """Max allocation limit configuration variable name"""

    @property
    def worker_thread(self) -> QuantConnect.Util.WorkerThread:
        """The worker thread instance the setup handler should use"""
        ...

    @property.setter
    def worker_thread(self, value: QuantConnect.Util.WorkerThread) -> None:
        ...

    @property
    def errors(self) -> System.Collections.Generic.List[System.Exception]:
        """Any errors from the initialization stored here:"""
        ...

    @property.setter
    def errors(self, value: System.Collections.Generic.List[System.Exception]) -> None:
        ...

    @property
    def maximum_runtime(self) -> datetime.timedelta:
        """Get the maximum runtime for this algorithm job."""
        ...

    @property
    def starting_portfolio_value(self) -> float:
        """Algorithm starting capital for statistics calculations"""
        ...

    @property
    def starting_date(self) -> datetime.datetime:
        """Start date for analysis loops to search for data."""
        ...

    @property
    def max_orders(self) -> int:
        """Maximum number of orders for the algorithm run -- applicable for backtests only."""
        ...

    def __init__(self) -> None:
        """Initializes a new BrokerageSetupHandler"""
        ...

    def create_algorithm_instance(self, algorithm_node_packet: QuantConnect.Packets.AlgorithmNodePacket, assembly_path: str) -> QuantConnect.Interfaces.IAlgorithm:
        """
        Create a new instance of an algorithm from a physical dll path.
        
        :param algorithm_node_packet: Details of the task required
        :param assembly_path: The path to the assembly's location
        :returns: A new instance of IAlgorithm, or throws an exception if there was an error.
        """
        ...

    def create_brokerage(self, algorithm_node_packet: QuantConnect.Packets.AlgorithmNodePacket, uninitialized_algorithm: QuantConnect.Interfaces.IAlgorithm, factory: typing.Optional[QuantConnect.Interfaces.IBrokerageFactory]) -> typing.Union[QuantConnect.Interfaces.IBrokerage, QuantConnect.Interfaces.IBrokerageFactory]:
        """
        Creates the brokerage as specified by the job packet
        
        :param algorithm_node_packet: Job packet
        :param uninitialized_algorithm: The algorithm instance before Initialize has been called
        :param factory: The brokerage factory
        :returns: The brokerage instance, or throws if error creating instance.
        """
        ...

    def dispose(self) -> None:
        """Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources."""
        ...

    def get_open_orders(self, algorithm: QuantConnect.Interfaces.IAlgorithm, result_handler: QuantConnect.Lean.Engine.Results.IResultHandler, transaction_handler: QuantConnect.Lean.Engine.TransactionHandlers.ITransactionHandler, brokerage: QuantConnect.Interfaces.IBrokerage) -> None:
        """
        Get the open orders from a brokerage. Adds Orders.Order and Orders.OrderTicket to the transaction handler
        
        This method is protected.
        
        :param algorithm: Algorithm instance
        :param result_handler: The configured result handler
        :param transaction_handler: The configurated transaction handler
        :param brokerage: Brokerage output instance
        """
        ...

    def load_existing_holdings_and_orders(self, brokerage: QuantConnect.Interfaces.IBrokerage, algorithm: QuantConnect.Interfaces.IAlgorithm, parameters: QuantConnect.Lean.Engine.Setup.SetupHandlerParameters) -> bool:
        """
        Loads existing holdings and orders
        
        This method is protected.
        """
        ...

    def setup(self, parameters: QuantConnect.Lean.Engine.Setup.SetupHandlerParameters) -> bool:
        """
        Primary entry point to setup a new algorithm
        
        :param parameters: The parameters object to use
        :returns: True on successfully setting up the algorithm state, or false on error.
        """
        ...


class BaseSetupHandler(System.Object):
    """
    Base class that provides shared code for
    the ISetupHandler implementations
    """

    ALGORITHM_CREATION_TIMEOUT: datetime.timedelta
    """Get the maximum time that the creation of an algorithm can take"""

    @staticmethod
    def get_configured_data_feeds() -> System.Collections.Generic.Dictionary[QuantConnect.SecurityType, System.Collections.Generic.List[QuantConnect.TickType]]:
        """Get the available data feeds from config.json,"""
        ...

    @staticmethod
    def initialize_debugging(algorithm_node_packet: QuantConnect.Packets.AlgorithmNodePacket, worker_thread: QuantConnect.Util.WorkerThread) -> bool:
        """
        Initialize the debugger
        
        :param algorithm_node_packet: The algorithm node packet
        :param worker_thread: The worker thread instance to use
        """
        ...

    @staticmethod
    def load_backtest_job_account_currency(algorithm: QuantConnect.Interfaces.IAlgorithm, job: QuantConnect.Packets.BacktestNodePacket) -> None:
        """Sets the account currency the algorithm should use if set in the job packet"""
        ...

    @staticmethod
    def load_backtest_job_cash_amount(algorithm: QuantConnect.Interfaces.IAlgorithm, job: QuantConnect.Packets.BacktestNodePacket) -> None:
        """Sets the initial cash for the algorithm if set in the job packet."""
        ...

    @staticmethod
    def set_brokerage_trading_day_per_year(algorithm: QuantConnect.Interfaces.IAlgorithm) -> None:
        """
        Set the number of trading days per year based on the specified brokerage model.
        
        :param algorithm: The algorithm instance
        :returns: The number of trading days per year. For specific brokerages (Coinbase, Binance, Bitfinex, Bybit, FTX, Kraken), the value is 365. For other brokerages, the default value is 252.
        """
        ...

    @staticmethod
    def setup(parameters: QuantConnect.Lean.Engine.Setup.SetupHandlerParameters) -> bool:
        """
        Primary entry point to setup a new algorithm
        
        :param parameters: The parameters object to use
        :returns: True on successfully setting up the algorithm state, or false on error.
        """
        ...

    @staticmethod
    def setup_currency_conversions(algorithm: QuantConnect.Interfaces.IAlgorithm, universe_selection: QuantConnect.Lean.Engine.DataFeeds.UniverseSelection, currencies_to_update_white_list: System.Collections.Generic.IReadOnlyCollection[str] = None) -> None:
        """
        Will first check and add all the required conversion rate securities
        and later will seed an initial value to them.
        
        :param algorithm: The algorithm instance
        :param universe_selection: The universe selection instance
        :param currencies_to_update_white_list: If passed, the currencies in the CashBook that are contained in this list will be updated. By default, if not passed (null), all currencies in the cashbook without a properly set up currency conversion will be updated. This is not intended for actual algorithms but for tests or for this method to be used as a helper.
        """
        ...





from typing import overload
from enum import Enum
import abc
import typing
import warnings

import System
import System.Collections
import System.Reflection
import System.Runtime.Serialization
import System.Security
import System.Security.Permissions


class SecurityRuleSet(Enum):
    """This class has no documentation."""

    NONE = 0

    LEVEL_1 = 1

    LEVEL_2 = 2


class SuppressUnmanagedCodeSecurityAttribute(System.Attribute):
    """This class has no documentation."""

    def __init__(self) -> None:
        ...


class SecurityElement(System.Object):
    """This class has no documentation."""

    @property
    def tag(self) -> str:
        ...

    @property.setter
    def tag(self, value: str) -> None:
        ...

    @property
    def attributes(self) -> System.Collections.Hashtable:
        ...

    @property.setter
    def attributes(self, value: System.Collections.Hashtable) -> None:
        ...

    @property
    def text(self) -> str:
        ...

    @property.setter
    def text(self, value: str) -> None:
        ...

    @property
    def children(self) -> System.Collections.ArrayList:
        ...

    @property.setter
    def children(self, value: System.Collections.ArrayList) -> None:
        ...

    @overload
    def __init__(self, tag: str) -> None:
        ...

    @overload
    def __init__(self, tag: str, text: str) -> None:
        ...

    def add_attribute(self, name: str, value: str) -> None:
        ...

    def add_child(self, child: System.Security.SecurityElement) -> None:
        ...

    def attribute(self, name: str) -> str:
        ...

    def copy(self) -> System.Security.SecurityElement:
        ...

    def equal(self, other: System.Security.SecurityElement) -> bool:
        ...

    @staticmethod
    def escape(str: str) -> str:
        ...

    @staticmethod
    def from_string(xml: str) -> System.Security.SecurityElement:
        ...

    @staticmethod
    def is_valid_attribute_name(name: str) -> bool:
        ...

    @staticmethod
    def is_valid_attribute_value(value: str) -> bool:
        ...

    @staticmethod
    def is_valid_tag(tag: str) -> bool:
        ...

    @staticmethod
    def is_valid_text(text: str) -> bool:
        ...

    def search_for_child_by_tag(self, tag: str) -> System.Security.SecurityElement:
        ...

    def search_for_text_of_tag(self, tag: str) -> str:
        ...

    def to_string(self) -> str:
        ...


class ISecurityEncodable(metaclass=abc.ABCMeta):
    """This class has no documentation."""

    def from_xml(self, e: System.Security.SecurityElement) -> None:
        ...

    def to_xml(self) -> System.Security.SecurityElement:
        ...


class IStackWalk(metaclass=abc.ABCMeta):
    """This class has no documentation."""

    def Assert(self) -> None:
        ...

    def demand(self) -> None:
        ...

    def deny(self) -> None:
        ...

    def permit_only(self) -> None:
        ...


class IPermission(System.Security.ISecurityEncodable, metaclass=abc.ABCMeta):
    """Obsoletions.CodeAccessSecurityMessage"""

    def copy(self) -> System.Security.IPermission:
        ...

    def demand(self) -> None:
        ...

    def intersect(self, target: System.Security.IPermission) -> System.Security.IPermission:
        ...

    def is_subset_of(self, target: System.Security.IPermission) -> bool:
        ...

    def union(self, target: System.Security.IPermission) -> System.Security.IPermission:
        ...


class PermissionSet(System.Object, System.Collections.ICollection, System.Runtime.Serialization.IDeserializationCallback, System.Security.ISecurityEncodable, System.Security.IStackWalk):
    """This class has no documentation."""

    @property
    def count(self) -> int:
        ...

    @property
    def is_read_only(self) -> bool:
        ...

    @property
    def is_synchronized(self) -> bool:
        ...

    @property
    def sync_root(self) -> System.Object:
        ...

    @overload
    def __init__(self, state: System.Security.Permissions.PermissionState) -> None:
        ...

    @overload
    def __init__(self, permSet: System.Security.PermissionSet) -> None:
        ...

    def add_permission(self, perm: System.Security.IPermission) -> System.Security.IPermission:
        ...

    def add_permission_impl(self, perm: System.Security.IPermission) -> System.Security.IPermission:
        """This method is protected."""
        ...

    def Assert(self) -> None:
        ...

    def contains_non_code_access_permissions(self) -> bool:
        ...

    @staticmethod
    def convert_permission_set(in_format: str, in_data: typing.List[int], out_format: str) -> typing.List[int]:
        """This member is marked as obsolete."""
        warnings.warn("This member is marked as obsolete.", DeprecationWarning)

    def copy(self) -> System.Security.PermissionSet:
        ...

    def copy_to(self, array: System.Array, index: int) -> None:
        ...

    def demand(self) -> None:
        ...

    def deny(self) -> None:
        """This member is marked as obsolete."""
        warnings.warn("This member is marked as obsolete.", DeprecationWarning)

    def equals(self, o: typing.Any) -> bool:
        ...

    def from_xml(self, et: System.Security.SecurityElement) -> None:
        ...

    def get_enumerator(self) -> System.Collections.IEnumerator:
        ...

    def get_enumerator_impl(self) -> System.Collections.IEnumerator:
        """This method is protected."""
        ...

    def get_hash_code(self) -> int:
        ...

    def get_permission(self, perm_class: typing.Type) -> System.Security.IPermission:
        ...

    def get_permission_impl(self, perm_class: typing.Type) -> System.Security.IPermission:
        """This method is protected."""
        ...

    def intersect(self, other: System.Security.PermissionSet) -> System.Security.PermissionSet:
        ...

    def is_empty(self) -> bool:
        ...

    def is_subset_of(self, target: System.Security.PermissionSet) -> bool:
        ...

    def is_unrestricted(self) -> bool:
        ...

    def permit_only(self) -> None:
        ...

    def remove_permission(self, perm_class: typing.Type) -> System.Security.IPermission:
        ...

    def remove_permission_impl(self, perm_class: typing.Type) -> System.Security.IPermission:
        """This method is protected."""
        ...

    @staticmethod
    def revert_assert() -> None:
        ...

    def set_permission(self, perm: System.Security.IPermission) -> System.Security.IPermission:
        ...

    def set_permission_impl(self, perm: System.Security.IPermission) -> System.Security.IPermission:
        """This method is protected."""
        ...

    def to_string(self) -> str:
        ...

    def to_xml(self) -> System.Security.SecurityElement:
        ...

    def union(self, other: System.Security.PermissionSet) -> System.Security.PermissionSet:
        ...


class SecurityCriticalScope(Enum):
    """SecurityCriticalScope is only used for .NET 2.0 transparency compatibility."""

    EXPLICIT = 0

    EVERYTHING = ...


class SecurityCriticalAttribute(System.Attribute):
    """This class has no documentation."""

    @property
    def scope(self) -> System.Security.SecurityCriticalScope:
        """SecurityCriticalScope is only used for .NET 2.0 transparency compatibility."""
        warnings.warn("SecurityCriticalScope is only used for .NET 2.0 transparency compatibility.", DeprecationWarning)

    @overload
    def __init__(self) -> None:
        ...

    @overload
    def __init__(self, scope: System.Security.SecurityCriticalScope) -> None:
        ...


class SecuritySafeCriticalAttribute(System.Attribute):
    """This class has no documentation."""

    def __init__(self) -> None:
        ...


class SecureString(System.Object, System.IDisposable):
    """This class has no documentation."""

    @property
    def length(self) -> int:
        ...

    @overload
    def __init__(self) -> None:
        ...

    @overload
    def __init__(self, value: typing.Any, length: int) -> None:
        ...

    def append_char(self, c: str) -> None:
        ...

    def clear(self) -> None:
        ...

    def copy(self) -> System.Security.SecureString:
        ...

    def dispose(self) -> None:
        ...

    def insert_at(self, index: int, c: str) -> None:
        ...

    def is_read_only(self) -> bool:
        ...

    def make_read_only(self) -> None:
        ...

    def remove_at(self, index: int) -> None:
        ...

    def set_at(self, index: int, c: str) -> None:
        ...


class SecurityRulesAttribute(System.Attribute):
    """This class has no documentation."""

    @property
    def skip_verification_in_full_trust(self) -> bool:
        ...

    @property.setter
    def skip_verification_in_full_trust(self, value: bool) -> None:
        ...

    @property
    def rule_set(self) -> System.Security.SecurityRuleSet:
        ...

    def __init__(self, ruleSet: System.Security.SecurityRuleSet) -> None:
        ...


class VerificationException(System.SystemException):
    """This class has no documentation."""

    @overload
    def __init__(self) -> None:
        ...

    @overload
    def __init__(self, message: str) -> None:
        ...

    @overload
    def __init__(self, message: str, innerException: System.Exception) -> None:
        ...

    @overload
    def __init__(self, info: System.Runtime.Serialization.SerializationInfo, context: System.Runtime.Serialization.StreamingContext) -> None:
        """
        This method is protected.
        
        Obsoletions.LegacyFormatterImplMessage
        """
        ...


class SecurityTransparentAttribute(System.Attribute):
    """This class has no documentation."""

    def __init__(self) -> None:
        ...


class SecurityException(System.SystemException):
    """This class has no documentation."""

    @property
    def demanded(self) -> System.Object:
        ...

    @property.setter
    def demanded(self, value: System.Object) -> None:
        ...

    @property
    def deny_set_instance(self) -> System.Object:
        ...

    @property.setter
    def deny_set_instance(self, value: System.Object) -> None:
        ...

    @property
    def failed_assembly_info(self) -> System.Reflection.AssemblyName:
        ...

    @property.setter
    def failed_assembly_info(self, value: System.Reflection.AssemblyName) -> None:
        ...

    @property
    def granted_set(self) -> str:
        ...

    @property.setter
    def granted_set(self, value: str) -> None:
        ...

    @property
    def method(self) -> System.Reflection.MethodInfo:
        ...

    @property.setter
    def method(self, value: System.Reflection.MethodInfo) -> None:
        ...

    @property
    def permission_state(self) -> str:
        ...

    @property.setter
    def permission_state(self, value: str) -> None:
        ...

    @property
    def permission_type(self) -> typing.Type:
        ...

    @property.setter
    def permission_type(self, value: typing.Type) -> None:
        ...

    @property
    def permit_only_set_instance(self) -> System.Object:
        ...

    @property.setter
    def permit_only_set_instance(self, value: System.Object) -> None:
        ...

    @property
    def refused_set(self) -> str:
        ...

    @property.setter
    def refused_set(self, value: str) -> None:
        ...

    @property
    def url(self) -> str:
        ...

    @property.setter
    def url(self, value: str) -> None:
        ...

    @overload
    def __init__(self) -> None:
        ...

    @overload
    def __init__(self, message: str) -> None:
        ...

    @overload
    def __init__(self, message: str, inner: System.Exception) -> None:
        ...

    @overload
    def __init__(self, message: str, type: typing.Type) -> None:
        ...

    @overload
    def __init__(self, message: str, type: typing.Type, state: str) -> None:
        ...

    @overload
    def __init__(self, info: System.Runtime.Serialization.SerializationInfo, context: System.Runtime.Serialization.StreamingContext) -> None:
        """
        This method is protected.
        
        Obsoletions.LegacyFormatterImplMessage
        """
        ...

    def get_object_data(self, info: System.Runtime.Serialization.SerializationInfo, context: System.Runtime.Serialization.StreamingContext) -> None:
        """Obsoletions.LegacyFormatterImplMessage"""
        warnings.warn("Obsoletions.LegacyFormatterImplMessage", DeprecationWarning)

    def to_string(self) -> str:
        ...


class SecurityTreatAsSafeAttribute(System.Attribute):
    """SecurityTreatAsSafe is only used for .NET 2.0 transparency compatibility. Use the SecuritySafeCriticalAttribute instead."""

    def __init__(self) -> None:
        ...


class PartialTrustVisibilityLevel(Enum):
    """This class has no documentation."""

    VISIBLE_TO_ALL_HOSTS = 0

    NOT_VISIBLE_BY_DEFAULT = 1


class AllowPartiallyTrustedCallersAttribute(System.Attribute):
    """This class has no documentation."""

    @property
    def partial_trust_visibility_level(self) -> System.Security.PartialTrustVisibilityLevel:
        ...

    @property.setter
    def partial_trust_visibility_level(self, value: System.Security.PartialTrustVisibilityLevel) -> None:
        ...

    def __init__(self) -> None:
        ...


class UnverifiableCodeAttribute(System.Attribute):
    """This class has no documentation."""

    def __init__(self) -> None:
        ...





from typing import overload
from enum import Enum
import datetime
import typing

import System
import System.Collections.Generic
import System.Diagnostics
import System.Reflection

System_Diagnostics_Debug_AppendFormatted_AssertInterpolatedStringHandler_T = typing.TypeVar("System_Diagnostics_Debug_AppendFormatted_AssertInterpolatedStringHandler_T")
System_Diagnostics_Debug_AppendFormatted_WriteIfInterpolatedStringHandler_T = typing.TypeVar("System_Diagnostics_Debug_AppendFormatted_WriteIfInterpolatedStringHandler_T")


class DebuggerNonUserCodeAttribute(System.Attribute):
    """This class has no documentation."""

    def __init__(self) -> None:
        ...


class DebugProvider(System.Object):
    """Provides default implementation for Write and Fail methods in Debug class."""

    def fail(self, message: str, detail_message: str) -> None:
        ...

    @staticmethod
    @overload
    def fail_core(stack_trace: str, message: str, detail_message: str, error_source: str) -> None:
        ...

    @staticmethod
    @overload
    def fail_core(stack_trace: str, message: str, detail_message: str, error_source: str) -> None:
        ...

    def on_indent_level_changed(self, indent_level: int) -> None:
        ...

    def on_indent_size_changed(self, indent_size: int) -> None:
        ...

    def write(self, message: str) -> None:
        ...

    @staticmethod
    @overload
    def write_core(message: str) -> None:
        ...

    @staticmethod
    @overload
    def write_core(message: str) -> None:
        ...

    def write_line(self, message: str) -> None:
        ...


class DebuggerDisplayAttribute(System.Attribute):
    """This class has no documentation."""

    @property
    def value(self) -> str:
        ...

    @property
    def name(self) -> str:
        ...

    @property.setter
    def name(self, value: str) -> None:
        ...

    @property
    def type(self) -> str:
        ...

    @property.setter
    def type(self, value: str) -> None:
        ...

    @property
    def target(self) -> typing.Type:
        ...

    @property.setter
    def target(self, value: typing.Type) -> None:
        ...

    @property
    def target_type_name(self) -> str:
        ...

    @property.setter
    def target_type_name(self, value: str) -> None:
        ...

    def __init__(self, value: str) -> None:
        ...


class StackTraceHiddenAttribute(System.Attribute):
    """
    Types and Methods attributed with StackTraceHidden will be omitted from the stack trace text shown in StackTrace.ToString()
    and Exception.StackTrace
    """

    def __init__(self) -> None:
        """Initializes a new instance of the StackTraceHiddenAttribute class."""
        ...


class ConditionalAttribute(System.Attribute):
    """This class has no documentation."""

    @property
    def condition_string(self) -> str:
        ...

    def __init__(self, conditionString: str) -> None:
        ...


class StackFrame(System.Object):
    """There is no good reason for the methods of this class to be virtual."""

    OFFSET_UNKNOWN: int = -1
    """Constant returned when the native or IL offset is unknown"""

    @overload
    def __init__(self) -> None:
        """Constructs a StackFrame corresponding to the active stack frame."""
        ...

    @overload
    def __init__(self, needFileInfo: bool) -> None:
        """Constructs a StackFrame corresponding to the active stack frame."""
        ...

    @overload
    def __init__(self, skipFrames: int) -> None:
        """Constructs a StackFrame corresponding to a calling stack frame."""
        ...

    @overload
    def __init__(self, skipFrames: int, needFileInfo: bool) -> None:
        """Constructs a StackFrame corresponding to a calling stack frame."""
        ...

    @overload
    def __init__(self, fileName: str, lineNumber: int) -> None:
        """
        Constructs a "fake" stack frame, just containing the given file
        name and line number.  Use when you don't want to use the
        debugger's line mapping logic.
        """
        ...

    @overload
    def __init__(self, fileName: str, lineNumber: int, colNumber: int) -> None:
        """
        Constructs a "fake" stack frame, just containing the given file
        name, line number and column number.  Use when you don't want to
        use the debugger's line mapping logic.
        """
        ...

    def get_file_column_number(self) -> int:
        """
        Returns the column number in the line containing the code being executed.
        This information is normally extracted from the debugging symbols
        for the executable.
        """
        ...

    def get_file_line_number(self) -> int:
        """
        Returns the line number in the file containing the code being executed.
        This information is normally extracted from the debugging symbols
        for the executable.
        """
        ...

    def get_file_name(self) -> str:
        """
        Returns the file name containing the code being executed.  This
        information is normally extracted from the debugging symbols
        for the executable.
        """
        ...

    def get_il_offset(self) -> int:
        """
        Returns the offset from the start of the IL code for the
        method being executed.  This offset may be approximate depending
        on whether the jitter is generating debuggable code or not.
        """
        ...

    def get_method(self) -> System.Reflection.MethodBase:
        ...

    def get_native_offset(self) -> int:
        """
        Returns the offset from the start of the native (jitted) code for the
        method being executed
        """
        ...

    def to_string(self) -> str:
        """Builds a readable representation of the stack frame"""
        ...


class DiagnosticMethodInfo(System.Object):
    """
    Represents diagnostic information about a method. Information provided by this class is similar to information
    provided by MethodBase but it's meant for logging and tracing purposes.
    """

    @property
    def name(self) -> str:
        """Gets the name of the method."""
        ...

    @property
    def declaring_type_name(self) -> str:
        """Gets the fully qualified name of the type that owns this method, including its namespace but not its assembly."""
        ...

    @property
    def declaring_assembly_name(self) -> str:
        """Gets the display name of the assembly that owns this method."""
        ...

    @staticmethod
    @overload
    def create(delegate: System.Delegate) -> System.Diagnostics.DiagnosticMethodInfo:
        """Creates a DiagnosticMethodInfo that represents the target of the delegate."""
        ...

    @staticmethod
    @overload
    def create(frame: System.Diagnostics.StackFrame) -> System.Diagnostics.DiagnosticMethodInfo:
        """Creates a DiagnosticMethodInfo that represents the method this stack frame is associtated with."""
        ...


class DebuggerHiddenAttribute(System.Attribute):
    """This class has no documentation."""

    def __init__(self) -> None:
        ...


class Debugger(System.Object):
    """This class has no documentation."""

    @staticmethod
    def break_for_user_unhandled_exception(exception: System.Exception) -> None:
        """
        Signals a breakpoint to an attached debugger with the  details
        if a .NET debugger is attached with break on user-unhandled exception enabled and a method
        attributed with DebuggerDisableUserUnhandledExceptionsAttribute calls this method.
        
        :param exception: The user-unhandled exception.
        """
        ...


class StackFrameExtensions(System.Object):
    """This class has no documentation."""

    @staticmethod
    def get_native_image_base(stack_frame: System.Diagnostics.StackFrame) -> System.IntPtr:
        ...

    @staticmethod
    def get_native_ip(stack_frame: System.Diagnostics.StackFrame) -> System.IntPtr:
        ...

    @staticmethod
    def has_il_offset(stack_frame: System.Diagnostics.StackFrame) -> bool:
        ...

    @staticmethod
    def has_method(stack_frame: System.Diagnostics.StackFrame) -> bool:
        ...

    @staticmethod
    def has_native_image(stack_frame: System.Diagnostics.StackFrame) -> bool:
        ...

    @staticmethod
    def has_source(stack_frame: System.Diagnostics.StackFrame) -> bool:
        ...


class DebuggerTypeProxyAttribute(System.Attribute):
    """This class has no documentation."""

    @property
    def proxy_type_name(self) -> str:
        ...

    @property
    def target(self) -> typing.Type:
        ...

    @property.setter
    def target(self, value: typing.Type) -> None:
        ...

    @property
    def target_type_name(self) -> str:
        ...

    @property.setter
    def target_type_name(self, value: str) -> None:
        ...

    @overload
    def __init__(self, type: typing.Type) -> None:
        ...

    @overload
    def __init__(self, typeName: str) -> None:
        ...


class DebuggerStepperBoundaryAttribute(System.Attribute):
    """Indicates the code following the attribute is to be executed in run, not step, mode."""

    def __init__(self) -> None:
        ...


class DebuggerDisableUserUnhandledExceptionsAttribute(System.Attribute):
    """
    If a .NET Debugger is attached which supports the Debugger.BreakForUserUnhandledException(Exception) API,
    this attribute will prevent the debugger from breaking on user-unhandled exceptions when the
    exception is caught by a method with this attribute, unless BreakForUserUnhandledException is called.
    """


class Stopwatch(System.Object):
    """This class has no documentation."""

    FREQUENCY: int = ...

    IS_HIGH_RESOLUTION: bool = True

    @property
    def is_running(self) -> bool:
        ...

    @property
    def elapsed(self) -> datetime.timedelta:
        ...

    @property
    def elapsed_milliseconds(self) -> int:
        ...

    @property
    def elapsed_ticks(self) -> int:
        ...

    def __init__(self) -> None:
        ...

    @staticmethod
    @overload
    def get_elapsed_time(starting_timestamp: int) -> datetime.timedelta:
        """
        Gets the elapsed time since the  value retrieved using GetTimestamp.
        
        :param starting_timestamp: The timestamp marking the beginning of the time period.
        :returns: A TimeSpan for the elapsed time between the starting timestamp and the time of this call.
        """
        ...

    @staticmethod
    @overload
    def get_elapsed_time(starting_timestamp: int, ending_timestamp: int) -> datetime.timedelta:
        """
        Gets the elapsed time between two timestamps retrieved using GetTimestamp.
        
        :param starting_timestamp: The timestamp marking the beginning of the time period.
        :param ending_timestamp: The timestamp marking the end of the time period.
        :returns: A TimeSpan for the elapsed time between the starting and ending timestamps.
        """
        ...

    @staticmethod
    def get_timestamp() -> int:
        ...

    def reset(self) -> None:
        ...

    def restart(self) -> None:
        ...

    def start(self) -> None:
        ...

    @staticmethod
    def start_new() -> System.Diagnostics.Stopwatch:
        ...

    def stop(self) -> None:
        ...

    def to_string(self) -> str:
        """
        Returns the Elapsed time as a string.
        
        :returns: Elapsed time string in the same format used by TimeSpan.ToString().
        """
        ...


class DebuggerVisualizerAttribute(System.Attribute):
    """
    Signifies that the attributed type has a visualizer which is pointed
    to by the parameter type name strings.
    """

    @property
    def visualizer_object_source_type_name(self) -> str:
        ...

    @property
    def visualizer_type_name(self) -> str:
        ...

    @property
    def description(self) -> str:
        ...

    @property.setter
    def description(self, value: str) -> None:
        ...

    @property
    def target(self) -> typing.Type:
        ...

    @property.setter
    def target(self, value: typing.Type) -> None:
        ...

    @property
    def target_type_name(self) -> str:
        ...

    @property.setter
    def target_type_name(self, value: str) -> None:
        ...

    @overload
    def __init__(self, visualizerTypeName: str) -> None:
        ...

    @overload
    def __init__(self, visualizerTypeName: str, visualizerObjectSourceTypeName: str) -> None:
        ...

    @overload
    def __init__(self, visualizerTypeName: str, visualizerObjectSource: typing.Type) -> None:
        ...

    @overload
    def __init__(self, visualizer: typing.Type) -> None:
        ...

    @overload
    def __init__(self, visualizer: typing.Type, visualizerObjectSource: typing.Type) -> None:
        ...

    @overload
    def __init__(self, visualizer: typing.Type, visualizerObjectSourceTypeName: str) -> None:
        ...


class DebuggerBrowsableState(Enum):
    """This class has no documentation."""

    NEVER = 0

    COLLAPSED = 2

    ROOT_HIDDEN = 3


class DebuggerBrowsableAttribute(System.Attribute):
    """This class has no documentation."""

    @property
    def state(self) -> System.Diagnostics.DebuggerBrowsableState:
        ...

    def __init__(self, state: System.Diagnostics.DebuggerBrowsableState) -> None:
        ...


class UnreachableException(System.Exception):
    """Exception thrown when the program executes an instruction that was thought to be unreachable."""

    @overload
    def __init__(self) -> None:
        """Initializes a new instance of the UnreachableException class with the default error message."""
        ...

    @overload
    def __init__(self, message: str) -> None:
        """
        Initializes a new instance of the UnreachableException
        class with a specified error message.
        
        :param message: The error message that explains the reason for the exception.
        """
        ...

    @overload
    def __init__(self, message: str, innerException: System.Exception) -> None:
        """
        Initializes a new instance of the UnreachableException
        class with a specified error message and a reference to the inner exception that is the cause of
        this exception.
        
        :param message: The error message that explains the reason for the exception.
        :param innerException: The exception that is the cause of the current exception.
        """
        ...


class DebuggableAttribute(System.Attribute):
    """This class has no documentation."""

    class DebuggingModes(Enum):
        """This class has no documentation."""

        NONE = ...

        DEFAULT = ...

        DISABLE_OPTIMIZATIONS = ...

        IGNORE_SYMBOL_STORE_SEQUENCE_POINTS = ...

        ENABLE_EDIT_AND_CONTINUE = ...

    @property
    def is_jit_tracking_enabled(self) -> bool:
        ...

    @property
    def is_jit_optimizer_disabled(self) -> bool:
        ...

    @property
    def debugging_flags(self) -> System.Diagnostics.DebuggableAttribute.DebuggingModes:
        ...

    @overload
    def __init__(self, isJITTrackingEnabled: bool, isJITOptimizerDisabled: bool) -> None:
        ...

    @overload
    def __init__(self, modes: System.Diagnostics.DebuggableAttribute.DebuggingModes) -> None:
        ...


class DebuggerStepThroughAttribute(System.Attribute):
    """This class has no documentation."""

    def __init__(self) -> None:
        ...


class StackTrace(System.Object):
    """
    Class which represents a description of a stack trace
    There is no good reason for the methods of this class to be virtual.
    """

    METHODS_TO_SKIP: int = 0

    @property
    def frame_count(self) -> int:
        """Property to get the number of frames in the stack trace"""
        ...

    @overload
    def __init__(self) -> None:
        """Constructs a stack trace from the current location."""
        ...

    @overload
    def __init__(self, fNeedFileInfo: bool) -> None:
        """Constructs a stack trace from the current location."""
        ...

    @overload
    def __init__(self, skipFrames: int) -> None:
        """
        Constructs a stack trace from the current location, in a caller's
        frame
        """
        ...

    @overload
    def __init__(self, skipFrames: int, fNeedFileInfo: bool) -> None:
        """
        Constructs a stack trace from the current location, in a caller's
        frame
        """
        ...

    @overload
    def __init__(self, e: System.Exception) -> None:
        """Constructs a stack trace from the current location."""
        ...

    @overload
    def __init__(self, e: System.Exception, fNeedFileInfo: bool) -> None:
        """Constructs a stack trace from the current location."""
        ...

    @overload
    def __init__(self, e: System.Exception, skipFrames: int) -> None:
        """
        Constructs a stack trace from the current location, in a caller's
        frame
        """
        ...

    @overload
    def __init__(self, e: System.Exception, skipFrames: int, fNeedFileInfo: bool) -> None:
        """
        Constructs a stack trace from the current location, in a caller's
        frame
        """
        ...

    @overload
    def __init__(self, frame: System.Diagnostics.StackFrame) -> None:
        """
        Constructs a "fake" stack trace, just containing a single frame.
        Does not have the overhead of a full stack trace.
        """
        ...

    @overload
    def __init__(self, frames: System.Collections.Generic.IEnumerable[System.Diagnostics.StackFrame]) -> None:
        """
        Constructs a stack trace from a set of StackFrame objects
        
        :param frames: The set of stack frames that should be present in the stack trace
        """
        ...

    def get_frame(self, index: int) -> System.Diagnostics.StackFrame:
        """
        Returns a given stack frame.  Stack frames are numbered starting at
        zero, which is the last stack frame pushed.
        """
        ...

    def get_frames(self) -> typing.List[System.Diagnostics.StackFrame]:
        """
        Returns an array of all stack frames for this stacktrace.
        The array is ordered and sized such that GetFrames()[i] == GetFrame(i)
        The nth element of this array is the same as GetFrame(n).
        The length of the array is the same as FrameCount.
        """
        ...

    def to_string(self) -> str:
        """Builds a readable representation of the stack trace"""
        ...


class Debug(System.Object):
    """Provides a set of properties and methods for debugging code."""

    class AssertInterpolatedStringHandler:
        """Provides an interpolated string handler for Debug.Assert that only performs formatting if the assert fails."""

        def __init__(self, literalLength: int, formattedCount: int, condition: bool, shouldAppend: typing.Optional[bool]) -> typing.Union[None, bool]:
            """
            Creates an instance of the handler..
            
            :param literalLength: The number of constant characters outside of interpolation expressions in the interpolated string.
            :param formattedCount: The number of interpolation expressions in the interpolated string.
            :param condition: The condition Boolean passed to the Debug method.
            :param shouldAppend: A value indicating whether formatting should proceed.
            """
            ...

        @overload
        def append_formatted(self, value: System_Diagnostics_Debug_AppendFormatted_AssertInterpolatedStringHandler_T) -> None:
            """
            Writes the specified value to the handler.
            
            :param value: The value to write.
            """
            ...

        @overload
        def append_formatted(self, value: System_Diagnostics_Debug_AppendFormatted_AssertInterpolatedStringHandler_T, format: str) -> None:
            """
            Writes the specified value to the handler.
            
            :param value: The value to write.
            :param format: The format string.
            """
            ...

        @overload
        def append_formatted(self, value: System_Diagnostics_Debug_AppendFormatted_AssertInterpolatedStringHandler_T, alignment: int) -> None:
            """
            Writes the specified value to the handler.
            
            :param value: The value to write.
            :param alignment: Minimum number of characters that should be written for this value.  If the value is negative, it indicates left-aligned and the required minimum is the absolute value.
            """
            ...

        @overload
        def append_formatted(self, value: System_Diagnostics_Debug_AppendFormatted_AssertInterpolatedStringHandler_T, alignment: int, format: str) -> None:
            """
            Writes the specified value to the handler.
            
            :param value: The value to write.
            :param alignment: Minimum number of characters that should be written for this value.  If the value is negative, it indicates left-aligned and the required minimum is the absolute value.
            :param format: The format string.
            """
            ...

        @overload
        def append_formatted(self, value: System.ReadOnlySpan[str]) -> None:
            """
            Writes the specified character span to the handler.
            
            :param value: The span to write.
            """
            ...

        @overload
        def append_formatted(self, value: System.ReadOnlySpan[str], alignment: int = 0, format: str = None) -> None:
            """
            Writes the specified string of chars to the handler.
            
            :param value: The span to write.
            :param alignment: Minimum number of characters that should be written for this value.  If the value is negative, it indicates left-aligned and the required minimum is the absolute value.
            :param format: The format string.
            """
            ...

        @overload
        def append_formatted(self, value: str) -> None:
            """
            Writes the specified value to the handler.
            
            :param value: The value to write.
            """
            ...

        @overload
        def append_formatted(self, value: str, alignment: int = 0, format: str = None) -> None:
            """
            Writes the specified value to the handler.
            
            :param value: The value to write.
            :param alignment: Minimum number of characters that should be written for this value.  If the value is negative, it indicates left-aligned and the required minimum is the absolute value.
            :param format: The format string.
            """
            ...

        @overload
        def append_formatted(self, value: typing.Any, alignment: int = 0, format: str = None) -> None:
            """
            Writes the specified value to the handler.
            
            :param value: The value to write.
            :param alignment: Minimum number of characters that should be written for this value.  If the value is negative, it indicates left-aligned and the required minimum is the absolute value.
            :param format: The format string.
            """
            ...

        def append_literal(self, value: str) -> None:
            """
            Writes the specified string to the handler.
            
            :param value: The string to write.
            """
            ...

    class WriteIfInterpolatedStringHandler:
        """Provides an interpolated string handler for Debug.WriteIf and Debug.WriteLineIf that only performs formatting if the condition applies."""

        def __init__(self, literalLength: int, formattedCount: int, condition: bool, shouldAppend: typing.Optional[bool]) -> typing.Union[None, bool]:
            """
            Creates an instance of the handler..
            
            :param literalLength: The number of constant characters outside of interpolation expressions in the interpolated string.
            :param formattedCount: The number of interpolation expressions in the interpolated string.
            :param condition: The condition Boolean passed to the Debug method.
            :param shouldAppend: A value indicating whether formatting should proceed.
            """
            ...

        @overload
        def append_formatted(self, value: System_Diagnostics_Debug_AppendFormatted_WriteIfInterpolatedStringHandler_T) -> None:
            """
            Writes the specified value to the handler.
            
            :param value: The value to write.
            """
            ...

        @overload
        def append_formatted(self, value: System_Diagnostics_Debug_AppendFormatted_WriteIfInterpolatedStringHandler_T, format: str) -> None:
            """
            Writes the specified value to the handler.
            
            :param value: The value to write.
            :param format: The format string.
            """
            ...

        @overload
        def append_formatted(self, value: System_Diagnostics_Debug_AppendFormatted_WriteIfInterpolatedStringHandler_T, alignment: int) -> None:
            """
            Writes the specified value to the handler.
            
            :param value: The value to write.
            :param alignment: Minimum number of characters that should be written for this value.  If the value is negative, it indicates left-aligned and the required minimum is the absolute value.
            """
            ...

        @overload
        def append_formatted(self, value: System_Diagnostics_Debug_AppendFormatted_WriteIfInterpolatedStringHandler_T, alignment: int, format: str) -> None:
            """
            Writes the specified value to the handler.
            
            :param value: The value to write.
            :param alignment: Minimum number of characters that should be written for this value.  If the value is negative, it indicates left-aligned and the required minimum is the absolute value.
            :param format: The format string.
            """
            ...

        @overload
        def append_formatted(self, value: System.ReadOnlySpan[str]) -> None:
            """
            Writes the specified character span to the handler.
            
            :param value: The span to write.
            """
            ...

        @overload
        def append_formatted(self, value: System.ReadOnlySpan[str], alignment: int = 0, format: str = None) -> None:
            """
            Writes the specified string of chars to the handler.
            
            :param value: The span to write.
            :param alignment: Minimum number of characters that should be written for this value.  If the value is negative, it indicates left-aligned and the required minimum is the absolute value.
            :param format: The format string.
            """
            ...

        @overload
        def append_formatted(self, value: str) -> None:
            """
            Writes the specified value to the handler.
            
            :param value: The value to write.
            """
            ...

        @overload
        def append_formatted(self, value: str, alignment: int = 0, format: str = None) -> None:
            """
            Writes the specified value to the handler.
            
            :param value: The value to write.
            :param alignment: Minimum number of characters that should be written for this value.  If the value is negative, it indicates left-aligned and the required minimum is the absolute value.
            :param format: The format string.
            """
            ...

        @overload
        def append_formatted(self, value: typing.Any, alignment: int = 0, format: str = None) -> None:
            """
            Writes the specified value to the handler.
            
            :param value: The value to write.
            :param alignment: Minimum number of characters that should be written for this value.  If the value is negative, it indicates left-aligned and the required minimum is the absolute value.
            :param format: The format string.
            """
            ...

        def append_literal(self, value: str) -> None:
            """
            Writes the specified string to the handler.
            
            :param value: The string to write.
            """
            ...

    auto_flush: bool

    indent_level: int

    indent_size: int

    @staticmethod
    @overload
    def Assert(condition: bool) -> None:
        ...

    @staticmethod
    @overload
    def Assert(condition: bool, message: str = None) -> None:
        ...

    @staticmethod
    @overload
    def Assert(condition: bool, message: System.Diagnostics.Debug.AssertInterpolatedStringHandler) -> None:
        ...

    @staticmethod
    @overload
    def Assert(condition: bool, message: str, detailMessage: str) -> None:
        ...

    @staticmethod
    @overload
    def Assert(condition: bool, message: System.Diagnostics.Debug.AssertInterpolatedStringHandler, detailMessage: System.Diagnostics.Debug.AssertInterpolatedStringHandler) -> None:
        ...

    @staticmethod
    @overload
    def Assert(condition: bool, message: str, detailMessageFormat: str, *args: typing.Any) -> None:
        ...

    @staticmethod
    def close() -> None:
        ...

    @staticmethod
    @overload
    def fail(message: str) -> None:
        ...

    @staticmethod
    @overload
    def fail(message: str, detail_message: str) -> None:
        ...

    @staticmethod
    def flush() -> None:
        ...

    @staticmethod
    def indent() -> None:
        ...

    @staticmethod
    @overload
    def print(message: str) -> None:
        ...

    @staticmethod
    @overload
    def print(format: str, *args: typing.Any) -> None:
        ...

    @staticmethod
    def set_provider(provider: System.Diagnostics.DebugProvider) -> System.Diagnostics.DebugProvider:
        ...

    @staticmethod
    def unindent() -> None:
        ...

    @staticmethod
    @overload
    def write(message: str) -> None:
        ...

    @staticmethod
    @overload
    def write(value: typing.Any) -> None:
        ...

    @staticmethod
    @overload
    def write(message: str, category: str) -> None:
        ...

    @staticmethod
    @overload
    def write(value: typing.Any, category: str) -> None:
        ...

    @staticmethod
    @overload
    def write_if(condition: bool, message: str) -> None:
        ...

    @staticmethod
    @overload
    def write_if(condition: bool, message: System.Diagnostics.Debug.WriteIfInterpolatedStringHandler) -> None:
        ...

    @staticmethod
    @overload
    def write_if(condition: bool, value: typing.Any) -> None:
        ...

    @staticmethod
    @overload
    def write_if(condition: bool, message: str, category: str) -> None:
        ...

    @staticmethod
    @overload
    def write_if(condition: bool, message: System.Diagnostics.Debug.WriteIfInterpolatedStringHandler, category: str) -> None:
        ...

    @staticmethod
    @overload
    def write_if(condition: bool, value: typing.Any, category: str) -> None:
        ...

    @staticmethod
    @overload
    def write_line(message: str) -> None:
        ...

    @staticmethod
    @overload
    def write_line(value: typing.Any) -> None:
        ...

    @staticmethod
    @overload
    def write_line(value: typing.Any, category: str) -> None:
        ...

    @staticmethod
    @overload
    def write_line(format: str, *args: typing.Any) -> None:
        ...

    @staticmethod
    @overload
    def write_line(message: str, category: str) -> None:
        ...

    @staticmethod
    @overload
    def write_line_if(condition: bool, value: typing.Any) -> None:
        ...

    @staticmethod
    @overload
    def write_line_if(condition: bool, value: typing.Any, category: str) -> None:
        ...

    @staticmethod
    @overload
    def write_line_if(condition: bool, message: str) -> None:
        ...

    @staticmethod
    @overload
    def write_line_if(condition: bool, message: System.Diagnostics.Debug.WriteIfInterpolatedStringHandler) -> None:
        ...

    @staticmethod
    @overload
    def write_line_if(condition: bool, message: str, category: str) -> None:
        ...

    @staticmethod
    @overload
    def write_line_if(condition: bool, message: System.Diagnostics.Debug.WriteIfInterpolatedStringHandler, category: str) -> None:
        ...


